# Licensed Materials - Property of IBM
# 5724-T05
#
# (C) Copyright IBM Corp. 2003,2010. All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
## Lucent UMTS NODEB data hash module.
## Define all the Global Variables which will be used by all the
## subroutines.

use strict;

my $idxHash = {};
my $cntHash = {};
my ($keyName,$keyVal);
my $gDataFile;
my $gMeasDate;

  ### 20080916: RP9.1
  my $all_idx = {};

  $all_idx->{NodeBpm} =
  {
     create_idx_NodeB       => \&create_idx_NodeB,
     create_idx_UtranCell   => \&create_idx_UtranCell,
     create_idx_UtranCell2  => \&create_idx_UtranCell,
     ### RP9.4 20100112
     create_idx_UtranCell3  => \&create_idx_UtranCell
  };
  $all_idx->{OneBTSpm} =
  {
     create_idx_NodeB       => \&create_idx_NodeB,
     create_idx_UtranCell   => \&create_idx_UtranCell,
     create_idx_UtranCell2  => \&create_idx_UtranCell,
     ### RP9.4 20100112
     create_idx_UtranCell3  => \&create_idx_UtranCell
  };
  ### end


sub build_data_hash {

  my ($ppConfig, $output, $insep, $config) = @_;

  my ($file_content, $meas_time, $cnt_name, $cnt_value, $perlen, $moid, $moid_entity, $moid_value );
  my ($filetype, $RNC_id, $NodeB_id);

  my $create_jobidx_cache= {}; 
  # We have to create job idx to fit with the Prospect hierarchy,
  # $rewrite_jobidx_cache caches the results (!) so that we don't have to
  # call the create subroutine more than once for a given MOID.

  my $create_jobidx_or_not= {};
  # $create_jobidx_or_not keeps track of whether we succeeded in creating
  # a given MOID or not. 
  #
  # The value will be one of -
  # undef - this job idx has not been created.
  # "yes"      - this job idx has been created successfuly
  # "failed"   - this job idx has failed to be created

  my $moid_separator= $ppConfig->{CONFIG}{MOIDSEPARATOR};

  my $nodeb_label = "";
  my $skip_ahead = "yes";

  while ($file_content = <STDIN>) {
    chomp($file_content);

    my ($line_type, $rest)= $file_content =~ /^([^$insep]*)$insep(.*)$/;
    if ($line_type eq "FILENAME") {
      $skip_ahead= "no";

      my $file_name = $rest;

      ($filetype, $RNC_id, $NodeB_id, $meas_time) = parse_filename($file_name, $config, $ppConfig);

      if (not defined $RNC_id or not defined $NodeB_id or not defined $meas_time)
      {
         # No point parsing this file if we cannot extract the RNC ID or the MEAS_TIME
	 messageHandler($config, 1, "Skipping ...");
         $skip_ahead="yes";
      } # endif not defined meas_time
      $nodeb_label = "";

    } elsif ($line_type eq "SN") {
      # Can get timestamp from SN but we do not appear to need it
    } elsif ($line_type eq "LABEL") {
      $nodeb_label = $rest;
      $nodeb_label =~ tr/!/|/;
    } else {
      # $line_type is not "SN" or "FILENAME"
      next if $skip_ahead eq "yes";
      if ($line_type eq "MOID") {
        ($moid, $perlen)= split /$insep/, $rest;
        ### 20080916: RP9.1
        my ($mid1, $mid2) = $moid =~ /(.*),(.*)/;
        $moid = (defined $mid2) ? $mid2 : $moid;
        ### End
        ($moid_entity, $moid_value) = $moid =~ /(\w+)=(.*)/;
      } else {
	($cnt_name,$cnt_value)=split(/$insep/,$file_content);
	# If the first field on the line is not "SN" or "MOID", 
	# it must be a counter name.

        my $jobidx;

        if (exists $ppConfig->{COUNTER_TO_GROUP}{$moid_entity} &&
            exists $ppConfig->{COUNTER_TO_GROUP}{$moid_entity}{$cnt_name} ) {
	    my $meas_group = $ppConfig->{COUNTER_TO_GROUP}{$moid_entity}{$cnt_name};

          ### 20090401: production ticket: if it happens that the same meas_group and moid_value,
          #             this will cause problem of create only ONE instance key for this create_jobidx_or_not.
          #             As such, we need to have the distinct key for both hash of 
          #             $create_jobidx_or_not and $create_jobidx_cache:
          #             with $RNC_id.$NodeB_id.$moid_value replacing only the $moid_value
          ### End 20090401

	  if (not exists $create_jobidx_or_not->{$meas_group} or
              not exists $create_jobidx_or_not->{$meas_group}{$RNC_id.$NodeB_id.$moid_value}) {

	    # Assume that we can't create the job idx until we've succeeded
	    # in rewriting it.
	    $create_jobidx_or_not->{$meas_group}{$RNC_id.$NodeB_id.$moid_value} = "failed";

            my $module = $ppConfig->{MODLIST}{$meas_group};
	    messageHandler($config, 3, "modList for $meas_group is ", $module);

            # 20080916: RP9.1 - use reference to functions.
            $jobidx = &{ $all_idx->{$filetype}{$module} } ($config, $RNC_id, $NodeB_id, $moid_value);

            if (defined $jobidx) {
              $create_jobidx_or_not->{$meas_group}{$RNC_id.$NodeB_id.$moid_value} = "done";
              # Place it in the cahce so we can utilise it next time
              $create_jobidx_cache->{$meas_group}{$RNC_id.$NodeB_id.$moid_value} = $jobidx;

              # Add the NodeB Label to the group.
              # It is done here so we only do it once
              if ($meas_group eq "NodeB") {
                my $field_pos=$ppConfig->{COUNTGROUPFIELD}{$meas_group}{'LABEL'};
                $output->{$meas_time}{$perlen}{$meas_group}{$jobidx}{$field_pos} = $nodeb_label;
              }

            } else {
              messageHandler($config, 1, "Could not create full key for counter : ", $cnt_name
                                       , "Measurement group ", $meas_group) 
            } # if defined $jobidx
	  } elsif ($create_jobidx_or_not->{$meas_group}{$RNC_id.$NodeB_id.$moid_value} eq "done") {
            $jobidx = $create_jobidx_cache->{$meas_group}{$RNC_id.$NodeB_id.$moid_value};
          } # if rewite jobidx
  
          if (defined $jobidx) {
            my $field_pos=$ppConfig->{COUNTGROUPFIELD}{$meas_group}{$cnt_name};
            $output->{$meas_time}{$perlen}{$meas_group}{$jobidx}{$field_pos} = $cnt_value;
          }
        } else {
          # Found counter in data which we do not know about
          messageHandler($config, 1, "No mapping found for counter : ",$cnt_name,", moid : ",$moid_entity);
        } # if COUNTER TO GROUP mapping 
      } # if $line_type eq "MOID"
    } # if $line_type eq "SN"
  } # while
} # sub build_data_hash

## --------------- End of build_data_hash sub routine--------

# initModules - provides any initialisation required for preLoadProcess modules
sub initModules {
  my ($rawDataFile, $config, $ppConfig) = @_ ;
  $ppConfig->{CONFIG} = $config ;
  $ppConfig->{ELEMENTLIST} = {} ;
  $ppConfig->{NODEID} = "unspecified" ;
  loadElementList($config,$ppConfig->{ELEMENTLIST});
  $ppConfig->{NODETYPE} = "RNC"; # Hack!
  loadPPConfig($config, $ppConfig) ;
  $config->{PPCONFIG} = $ppConfig ;
} # sub initModules
## -------- End of initModules SubRoutine---------------------


# initialiseConfiguration - setup the configuration parameters for this module
sub initialiseConfiguration {

  my %user_defaults=
  (
    # Regex for pulling year, month, day, hours, minutes and seconds out of
    # a timestamp.
    INPUT_TSTAMP_FORMAT =>
    "(\\d{4})(\\d{2})(\\d{2})(\\d{2})(\\d{2})(\\d{2})",

    # Format to use with (s)printf to output timestamps in the format we want.
    OUTPUT_TSTAMP_FORMAT =>
    "%04d%02d%02d%02d%02d%02d",

    LOGFILENAME => "preProcessLog" . todays_date(),
    );

  register_config_defaults (%user_defaults);

  register_read_only_variables
    ("LOGFILENAME");

  register_mandatory_variables
    ("LOGFILEDIR",
     "INSEP",
     "OUTSEP",
     "NODEBXMLPATTERN",
     "NODEBFILENAMEPATTERN",
     "ELEMENTLIST",
     "LOADCONFIGFILE",
    );
}
## -------- End of initialiseConfiguration SubRoutine---------------------



sub loadElementList {
  my ($config, $elementList) = @_ ;
  my ($fileName)= $config->{ELEMENTLIST};
  my ($line) ;
  if (!(open(THEFILE, "$fileName"))) {
    messageHandler ($config, 0, "Could not open element list ($fileName) for reading: $!") ;
    exit 1;
  }
  
  while (<THEFILE>) {
    chomp;
    next if /^\#/ or /^\s*$/; # Skip comments and blank lines.
    my ($element_name, $sys_id, $element_id, $release, $buff_size, $entity_type, $tech_type) = split ('\t', $_);

    if (not defined $tech_type) {
      messageHandler ($config, 1, "Element list ($fileName) corrupted for element : $element_name   Skipping ...");
      next;
    }

    if ($entity_type eq 'RNC') {
      if ($element_id ne '') {
        $elementList->{$element_name}{ID} = $element_id ;
      } else {
        messageHandler($config, 1, "No element ID defined in element list ($fileName) for element: $element_name  Skipping ...");
      }
    }
  }
  close(THEFILE);
} # sub loadElementList

##------------End of loadElementList module------------------

sub loadPPConfig {
  my ($config, $ppConfig) = @_ ;
  my $fileName= $config->{LOADCONFIGFILE};

  $ppConfig->{GROUPFIELDNAME} = {} ;
  $ppConfig->{REMMAPPING} = {} ; # Required only for remediation
  if (!(open(THEFILE, "$fileName"))) {
    messageHandler ($ppConfig, 0, "Could not open ppConfig file ($fileName) for reading: $!") ;
    exit 1 ;
  }

  my $line ;
  while ($line = <THEFILE>) {
    chomp $line ;
    next if ($line =~ /^\#/) ;
    next if ($line eq '') ;
    my ($counterNameData, $counterNameDD, $moid_entity, $measurementGroup,
	$groupField, $modList) = split(/\t/, $line);
    $modList =~ s/\"//g ; # Excel inserts quotes, so remove them
    if (length($counterNameData) > 0) {
      if (not defined $modList) {
        messageHandler($ppConfig, 1, "Invalid line found in group mapping file - skipping",
		       "Line : ", $line );
        next;
      }

      if ($modList ne "N/A") {
        if (exists $ppConfig->{MODLIST}{$measurementGroup} and
            $modList ne $ppConfig->{MODLIST}{$measurementGroup}) {
          messageHandler($ppConfig, 1, "Module list \"$modList\" for ",
                         "counter $counterNameDD is different ",
                         "from previous values for measurement group ",
                         "$measurementGroup - ignoring.");
        } 
        ### 20081007: RP9.1
        elsif ( (! exists $all_idx->{NodeBpm}{$modList} ) || 
                (! exists $all_idx->{OneBTSpm}{$modList} ))
        {
           messageHandler($ppConfig, 1, "Module list \"$modList\" for ",
                         "counter $counterNameDD does not defined.");
        }
        ### end
        else {
          $ppConfig->{MODLIST}{$measurementGroup}= $modList;
        }
      }

      if (!exists($ppConfig->{GROUPFIELDNAME}{$measurementGroup}{maxField}) ||
         ($groupField > $ppConfig->{GROUPFIELDNAME}{$measurementGroup}{maxField})) {
        $ppConfig->{GROUPFIELDNAME}{$measurementGroup}{maxField} = $groupField ;
      }

      if (exists ($ppConfig->{COUNTGROUPFIELD}{$measurementGroup}{$counterNameData})) {
        messageHandler ($ppConfig, 1, "Group field value $groupField for ",
                        "counter $counterNameDD duplicates a previous ",
                        "value in measurement group $measurementGroup.");
        # We still store it, because this is a way to handle counters
        # being renamed.
      }
      $ppConfig->{COUNTGROUPFIELD}{$measurementGroup}{$counterNameData} = $groupField;

      $ppConfig->{GROUPFIELDNAME}{$measurementGroup}{field}{$groupField}= $counterNameDD;

      $ppConfig->{COUNTER_TO_GROUP}{$moid_entity}{$counterNameData} = $measurementGroup;
      # This allows for the grouping of counters in the data to be different
      # from the grouping of counters in Prospect.
      $ppConfig->{SHORT_COUNTER_TO_GROUP}{$counterNameDD}= $measurementGroup;
    } # if length $counterNameData > 0
  } # while $line = <THEFILE>
  close(THEFILE);

} # sub loadPPConfig

## ------------End of loadPPConfig module-------------------


sub showGeneral {
  my ($fileName, $measTime, $perlen, $ppConfig, $config, $outsep) = @_ ;
  $measTime= round_tstamp_to_period($config, $measTime, $perlen);
  print join ($outsep,
	      "MEASGROUP_NodeB_General",
	      "5",
	      "FILENAME",
	      "TIMESTAMP",
	      "PERLENSEC",
	      "\n"
	     );
  print join ($outsep,
              "NODEB",
	      "",
	      $fileName,
	      $measTime,
	      $perlen,
	      "\n"
	     );
} # sub showGeneral

## ------------End of showGeneral SubRoutine---------------


## Subroutine to parse the file name extract the timestamp and RNC_ID
sub parse_filename {
  my ($file_name, $config, $ppConfig) = @_;

  ### 20080916 <RP9.1>
  my $all_filetypes = {
     NodeBpm => 0,
     OneBTSpm => 0,
  };

  my ($RNC_name, $RNC_id, $NodeB_id, $meas_time, $filetype);

  ($RNC_name, $NodeB_id, $meas_time, $filetype) = $file_name =~ /$config->{NODEBFILENAMEPATTERN}/;

  if ( ! exists ($all_filetypes->{$filetype}) )
  {
     messageHandler ($config, 1, "Unknown datatype for $file_name!");
     return (undef, undef, undef);
  }

  if (not defined $meas_time) {
    messageHandler ($config, 1, "Could not extract RNC name, NodeB id and meas time from filename.  Filename : ",$file_name);
    return (undef, undef, undef);
  }

  # Add seconds to the measurement time
  $meas_time = $meas_time . "00";

  if ($filetype eq 'NodeBpm') 
  {
    if (exists $ppConfig->{ELEMENTLIST}{$RNC_name}) {
      $RNC_id = $ppConfig->{ELEMENTLIST}{$RNC_name}{ID};
    } else {
      messageHandler ($config, 1, "Could not find RNC name in element list.  RNC name: ",$RNC_name);
      return (undef, undef, undef);
    }
  }
  elsif ($filetype eq 'OneBTSpm')
  {
    if (! defined $RNC_name)  {
       messageHandler ($config, 1, "Could not extract NodeB Id from filename.  Filename : ",$file_name);
       return (undef, undef, undef);
    }
    $RNC_id = $RNC_name; 
  }

  if (not defined $NodeB_id) {
    messageHandler ($config, 1, "Could not extract NodeB Id from filename.  Filename : ",$file_name);
    return (undef, undef, undef);
  }

  ### 20080916: RP9.1
  ### convert keys to uppercase  and '-' -> '_'
  $RNC_id   =~ tr/a-z/A-Z/; $RNC_id   =~ s/-/_/g;
  $NodeB_id =~ tr/a-z/A-Z/; $NodeB_id =~ s/-/_/g;

  return ($filetype, $RNC_id, $NodeB_id, $meas_time);
}
##--------End of parse_filename-----------------


### Subrouting to create NodeB full key
sub create_idx_NodeB {
  my ($config, $RNC_id, $NodeB_id, $moid_value) = @_;

  return "${RNC_id}-${NodeB_id}";
}
##---------End of create_idx_NodeB --------------


### Subrouting to create UtranCell full key
sub create_idx_UtranCell {
  my ($config, $RNC_id, $NodeB_id, $moid_value) = @_;
 
  if (not defined $moid_value or $moid_value eq '') {
    messageHandler($config, 1, "Could not create full key becuase moid value is undefined or null");
    return undef;
  }
  ### 20080916: RP9.1
  ### find_instance are aleways in UPPERCASE.
  ### So, it is safer to convert ids to UPPERCASE as well
  $moid_value =~ tr/a-z/A-Z/;

  return "${RNC_id}-${NodeB_id}-${moid_value}";
}
##---------End of create_idx_UtranCell ----------


##Return True.
1;
